@@ -1,6 +1,6 @@
 /*
 ** Splint - annotation-assisted static program checker
-** Copyright (C) 1994-2002 University of Virginia,
+** Copyright (C) 1994-2003 University of Virginia,
 **         Massachusetts Institute of Technology
 **
 ** This program is free software; you can redistribute it and/or modify it
@@ -50,7 +50,7 @@
 
 
 # include "splintMacros.nf"
-# include "llbasic.h"
+# include "basic.h"
 # include "gram.h"
 # include "lclscan.h"
 # include "scanline.h"
@@ -775,11 +775,11 @@ ScanCString (void)
 	    }
 	  else if (currentChar == '\n')
 	    {
-	      LocalUserError ("unterminated C string");
+	      LocalUserError ("Unterminated C string");
 	    }
 	  else
 	    {
-	      LocalUserError ("invalid character in C string");
+	      LocalUserError ("Invalid character in C string");
 	    }
 	}
       LCLMOVECHAR ();		/* discard closing quote */
@@ -822,11 +822,11 @@ ScanCChar (void)
 	    }
 	  else if (currentChar == '\n')
 	    {
-	      LocalUserError ("unterminated C character constant");
+	      LocalUserError ("Unterminated C character constant");
 	    }
 	  else
 	    {
-	      LocalUserError ("invalid character in C character");
+	      LocalUserError ("Invalid character in C character");
 	    }
 	}
       LCLMOVECHAR ();		/* discard closing quote */
@@ -834,7 +834,7 @@ ScanCChar (void)
     }
   else
     {
-      LocalUserError ("invalid C character");
+      LocalUserError ("Invalid C character");
     }
 
 
